require "rails_autobots/environment_rewriter"

class RailsAutobots::RewriteRuleset
  def initialize
    @rulesets = []
  end

  def add(environment_name, rewrite_function)
    @rulesets << RailsAutobots::EnvironmentRewriter.new(environment_name, rewrite_function)
  end

  def rewrite(markup)
    d = Nokogiri::HTML(markup)
    environments = environment_nodes(d)
    any_rewritten = false

    environments.each do |e|
      name = strip_environment_suffix(e["name"])
      content = JSON.parse(CGI.unescape(e["content"]))
      original_content = content.deep_dup
      @rulesets.each { |r| r.process(name, content) }

      if original_content != content
        any_rewritten = true
        e["content"] = CGI.escape(content.to_json)
      end
    end

    any_rewritten ? d.to_html : markup
  end

  def environment_nodes(d)
    d.css("meta[name$='/config/environment']")
  end

  def strip_environment_suffix(s)
    i = s.index("/")
    s[0...i]
  end
end
